/*
 *  CPAchecker is a tool for configurable software verification.
 *  This file is part of CPAchecker. 
 *
 *  Copyright (C) 2007-2008  Dirk Beyer and Erkan Keremoglu.
 *  All rights reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *
 *  CPAchecker web page:
 *    http://www.cs.sfu.ca/~dbeyer/CPAchecker/
 */
package cpa.assumptions.collector.progressobserver;

import java.util.Collection;

import com.google.common.base.Function;
import com.google.common.collect.Collections2;

import cfa.objectmodel.CFANode;

import cpa.common.UnmodifiableReachedElements;
import cpa.common.interfaces.AbstractElement;

/**
 * View of the reached heuristics
 * @author g.theoduloz
 */
public class ReachedHeuristicsDataSetView {
  
  private final UnmodifiableReachedElements underlying;
  private final Function<AbstractElement, StopHeuristicsData> mapFunction;
  
  public ReachedHeuristicsDataSetView(UnmodifiableReachedElements pUnderlying, Function<AbstractElement, StopHeuristicsData> pMapFunction)
  {
    underlying = pUnderlying;
    mapFunction = pMapFunction;
  }
  
  public Collection<StopHeuristicsData> getHeuristicsData()
  {
    return Collections2.transform(underlying.getReached(), mapFunction);
  }
  
  public Collection<StopHeuristicsData> getHeuristicsDataForLocation(CFANode location)
  {
    return Collections2.transform(underlying.getReached(location), mapFunction);    
  }
  
}
